@@ -1,4 +1,4 @@
-# Copyright (C) 2010-2018  Vincent Pelletier <plr.vincent@gmail.com>
+# Copyright (C) 2010-2021  Vincent Pelletier <plr.vincent@gmail.com>
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -53,24 +53,18 @@ from ctypes.util import find_library
 import sys
 import threading
 import warnings
-import weakref
+from weakref import WeakSet
 import collections
 import functools
 import contextlib
 import inspect
 from . import libusb1
-if sys.version_info[:2] >= (2, 6):
 # pylint: disable=wrong-import-order,ungrouped-imports
-    if sys.platform == 'win32':
-        from ctypes import get_last_error as get_errno
-    else:
-        from ctypes import get_errno
-# pylint: enable=wrong-import-order,ungrouped-imports
+if sys.platform == 'win32':
+    from ctypes import get_last_error as get_errno
 else:
-    def get_errno():
-        raise NotImplementedError(
-            'Your python version does not support errno/last_error'
-        )
+    from ctypes import get_errno
+# pylint: enable=wrong-import-order,ungrouped-imports
 
 __all__ = [
     'USBContext', 'USBDeviceHandle', 'USBDevice', 'hasCapability',
@@ -133,15 +127,10 @@ def mayRaiseUSBError(
         __raiseUSBError(value)
     return value
 
-try:
-    namedtuple = collections.namedtuple
-except AttributeError:
-    Version = tuple
-else:
-    Version = namedtuple(
-        'Version',
-        ['major', 'minor', 'micro', 'nano', 'rc', 'describe'],
-    )
+Version = collections.namedtuple(
+    'Version',
+    ['major', 'minor', 'micro', 'nano', 'rc', 'describe'],
+)
 
 if sys.version_info[0] == 3:
     BYTE = bytes([0])
@@ -216,29 +205,6 @@ else:
 CONTROL_SETUP = BYTE * CONTROL_SETUP_SIZE
 # pylint: enable=undefined-variable
 
-__libc_name = find_library('c')
-if __libc_name is None:
-    # Of course, will leak memory.
-    # Should we warn user ? How ?
-    _free = lambda x: None
-else:
-    _free = getattr(cdll, __libc_name).free
-del __libc_name
-
-try:
-    WeakSet = weakref.WeakSet
-except AttributeError:
-    # Python < 2.7: tiny wrapper around WeakKeyDictionary
-    class WeakSet(object):
-        def __init__(self):
-            self.__dict = weakref.WeakKeyDictionary()
-
-        def add(self, item):
-            self.__dict[item] = None
-
-        def pop(self):
-            return self.__dict.popitem()[0]
-
 # Default string length
 # From a comment in libusb-1.0: "Some devices choke on size > 255"
 STRING_LENGTH = 255
@@ -945,111 +911,6 @@ class USBTransferHelper(object):
         # Deprecated: to drop
         return self.__transfer.isSubmitted()
 
-# BBB
-class USBPollerThread(threading.Thread):
-    """
-    Implements libusb1 documentation about threaded, asynchronous
-    applications.
-    In short, instanciate this class once (...per USBContext instance), call
-    start() on the instance, and do whatever you need.
-    This thread will be used to execute transfer completion callbacks, and you
-    are free to use libusb1's synchronous API in another thread, and can forget
-    about libusb1 file descriptors.
-
-    See http://libusb.sourceforge.net/api-1.0/mtasync.html .
-    """
-    _can_run = True
-
-    def __init__(self, context, poller, exc_callback=None):
-        """
-        Create a poller thread for given context.
-        Warning: it will not check if another poller instance was already
-        present for that context, and will replace it.
-
-        poller
-            (same as USBPoller.__init__ "poller" parameter)
-
-        exc_callback (callable)
-          Called with a libusb_error value as single parameter when event
-          handling fails.
-          If not given, an USBError will be raised, interrupting the thread.
-        """
-        super(USBPollerThread, self).__init__()
-        warnings.warn(
-            'USBPollerThread causes long stalls when used with poll (it was '
-            'intended for epoll), and is generally misleading. Consider '
-            'calling looping over context.handleEvents() in a thread instead.',
-            DeprecationWarning,
-        )
-        self.daemon = True
-        self.__context = context
-        self.__poller = poller
-        self.__fd_set = set()
-        if exc_callback is not None:
-            self.exceptionHandler = exc_callback
-
-    def stop(self):
-        """
-        Stop & join thread.
-
-        Allows stopping event thread before context gets closed.
-        """
-        self._can_run = False
-        self.join()
-
-    # pylint: disable=method-hidden
-    @staticmethod
-    def exceptionHandler(exc):
-        raise exc
-    # pylint: enable=method-hidden
-
-    def run(self):
-        # We expect quite some spinning in below loop, so move any unneeded
-        # operation out of it.
-        context = self.__context
-        poll = self.__poller.poll
-        try_lock_events = context.tryLockEvents
-        lock_event_waiters = context.lockEventWaiters
-        wait_for_event = context.waitForEvent
-        unlock_event_waiters = context.unlockEventWaiters
-        event_handling_ok = context.eventHandlingOK
-        unlock_events = context.unlockEvents
-        handle_events_locked = context.handleEventsLocked
-        event_handler_active = context.eventHandlerActive
-        getNextTimeout = context.getNextTimeout
-        exceptionHandler = self.exceptionHandler
-        fd_set = self.__fd_set
-        context.setPollFDNotifiers(self._registerFD, self._unregisterFD)
-        for fd, events in context.getPollFDList():
-            self._registerFD(fd, events, None)
-        try:
-            while fd_set and self._can_run:
-                if try_lock_events():
-                    lock_event_waiters()
-                    while event_handler_active():
-                        wait_for_event()
-                    unlock_event_waiters()
-                else:
-                    try:
-                        while event_handling_ok():
-                            if poll(getNextTimeout()):
-                                try:
-                                    handle_events_locked()
-                                except USBError:
-                                    exceptionHandler(sys.exc_info()[1])
-                    finally:
-                        unlock_events()
-        finally:
-            context.setPollFDNotifiers(None, None)
-
-    def _registerFD(self, fd, events, _):
-        self.__poller.register(fd, events)
-        self.__fd_set.add(fd)
-
-    def _unregisterFD(self, fd, _):
-        self.__fd_set.discard(fd)
-        self.__poller.unregister(fd)
-
 class USBPoller(object):
     """
     Class allowing integration of USB event polling in a file-descriptor
@@ -1172,7 +1033,6 @@ class USBDeviceHandle(object):
     # pylint: enable=undefined-variable
     __set = set
     __KeyError = KeyError
-    __sys = sys
 
     def __init__(self, context, handle, device):
         """
@@ -1337,8 +1197,8 @@ class USBDeviceHandle(object):
         Reinitialise current device.
         Attempts to restore current configuration & alt settings.
         If this fails, will result in a device disconnect & reconnect, so you
-        have to close current device and rediscover it (notified by a
-        ERROR_NOT_FOUND error code).
+        have to close current device and rediscover it (notified by an
+        USBErrorNotFound exception).
         """
         mayRaiseUSBError(libusb1.libusb_reset_device(self.__handle))
 
@@ -1910,6 +1770,9 @@ class USBDevice(object):
                     continue
                 mayRaiseUSBError(result)
                 append(config.contents)
+        self.__bus_number = libusb1.libusb_get_bus_number(device_p)
+        self.__port_number = libusb1.libusb_get_port_number(device_p)
+        self.__device_address = libusb1.libusb_get_device_address(device_p)
 
     def __del__(self):
         self.close()
@@ -1952,9 +1815,9 @@ class USBDevice(object):
 
     def __key(self):
         return (
-            id(self.__context), self.getBusNumber(),
-            self.getDeviceAddress(), self.getVendorID(),
-            self.getProductID(),
+            id(self.__context), self.__bus_number,
+            self.__device_address, self.device_descriptor.idVendor,
+            self.device_descriptor.idProduct,
         )
 
     def __hash__(self):
@@ -1988,13 +1851,13 @@ class USBDevice(object):
         """
         Get device's bus number.
         """
-        return libusb1.libusb_get_bus_number(self.device_p)
+        return self.__bus_number
 
     def getPortNumber(self):
         """
         Get device's port number.
         """
-        return libusb1.libusb_get_port_number(self.device_p)
+        return self.__port_number
 
     def getPortNumberList(self):
         """
@@ -2013,7 +1876,7 @@ class USBDevice(object):
         """
         Get device's address on its bus.
         """
-        return libusb1.libusb_get_device_address(self.device_p)
+        return self.__device_address
 
     def getbcdUSB(self):
         """
@@ -2215,7 +2078,7 @@ class USBContext(object):
                 with self.__context_cond:
                     self.__context_refcount -= 1
                     if not self.__context_refcount:
-                        self.__context_cond.notifyAll()
+                        self.__context_cond.notify_all()
         if inspect.isgeneratorfunction(func):
             def wrapper(self, *args, **kw):
                 with refcount(self):
@@ -2293,13 +2156,14 @@ class USBContext(object):
                 self.__context_cond.wait()
             self._exit()
         finally:
-            self.__context_cond.notifyAll()
+            self.__context_cond.notify_all()
             self.__context_cond.release()
 
     def _exit(self):
         context_p = self.__context_p
         if context_p:
-            for handle in list(self.__hotplug_callback_dict.keys()):
+            while self.__hotplug_callback_dict:
+                handle = next(iter(self.__hotplug_callback_dict))
                 self.hotplugDeregisterCallback(handle)
             pop = self.__close_set.pop
             while True:
@@ -2429,7 +2293,7 @@ class USBContext(object):
                 ))
                 fd_index += 1
         finally:
-            _free(pollfd_p_p)
+            libusb1.libusb_free_pollfds(pollfd_p_p)
         return result
 
     @_validContext
@@ -2523,30 +2387,37 @@ class USBContext(object):
 
     @_validContext
     def tryLockEvents(self):
-        """
-        See libusb_try_lock_events doc.
-        """
+        warnings.warn(
+            'You may not be able to unlock in the event of USBContext exit. '
+            'Consider looping over handleEvents() in a thread.',
+            DeprecationWarning,
+        )
         return libusb1.libusb_try_lock_events(self.__context_p)
 
     @_validContext
     def lockEvents(self):
-        """
-        See libusb_lock_events doc.
-        """
+        warnings.warn(
+            'You may not be able to unlock in the event of USBContext exit. '
+            'Consider looping over handleEvents() in a thread.',
+            DeprecationWarning,
+        )
         libusb1.libusb_lock_events(self.__context_p)
 
     @_validContext
     def lockEventWaiters(self):
-        """
-        See libusb_lock_event_waiters doc.
-        """
+        warnings.warn(
+            'You may not be able to unlock in the event of USBContext exit. '
+            'Consider looping over handleEvents() in a thread.',
+            DeprecationWarning,
+        )
         libusb1.libusb_lock_event_waiters(self.__context_p)
 
     @_validContext
     def waitForEvent(self, tv=0):
-        """
-        See libusb_wait_for_event doc.
-        """
+        warnings.warn(
+            'Consider looping over handleEvents() in a thread.',
+            DeprecationWarning,
+        )
         if tv is None:
             tv = 0
         tv_s = int(tv)
@@ -2555,30 +2426,38 @@ class USBContext(object):
 
     @_validContext
     def unlockEventWaiters(self):
-        """
-        See libusb_unlock_event_waiters doc.
-        """
+        warnings.warn(
+            'This method will lock in the event of USBContext exit, '
+            'preventing libusb lock release. '
+            'Consider looping over handleEvents() in a thread.',
+            DeprecationWarning,
+        )
         libusb1.libusb_unlock_event_waiters(self.__context_p)
 
     @_validContext
     def eventHandlingOK(self):
-        """
-        See libusb_event_handling_ok doc.
-        """
+        warnings.warn(
+            'Consider looping over handleEvents() in a thread.',
+            DeprecationWarning,
+        )
         return libusb1.libusb_event_handling_ok(self.__context_p)
 
     @_validContext
     def unlockEvents(self):
-        """
-        See libusb_unlock_events doc.
-        """
+        warnings.warn(
+            'This method will lock in the event of USBContext exit, '
+            'preventing libusb lock release. '
+            'Consider looping over handleEvents() in a thread.',
+            DeprecationWarning,
+        )
         libusb1.libusb_unlock_events(self.__context_p)
 
     @_validContext
     def handleEventsLocked(self):
-        """
-        See libusb_handle_events_locked doc.
-        """
+        warnings.warn(
+            'Consider looping over handleEvents() in a thread.',
+            DeprecationWarning,
+        )
         # XXX: does tv parameter need to be exposed ?
         mayRaiseUSBError(libusb1.libusb_handle_events_locked(
             self.__context_p, _zero_tv_p,
@@ -2586,9 +2465,10 @@ class USBContext(object):
 
     @_validContext
     def eventHandlerActive(self):
-        """
-        See libusb_event_handler_active doc.
-        """
+        warnings.warn(
+            'Consider looping over handleEvents() in a thread.',
+            DeprecationWarning,
+        )
         return libusb1.libusb_event_handler_active(self.__context_p)
 
     @staticmethod
